How to install and setup the ExpressPlay Marlin DRM plugin for Wowza
====================================================================


Introduction
------------

The ExpressPlay Marlin DRM plugin for Wowza integrates directly with the ExpressPlay Key Storage interface (http://www.expressplay.com/developer#keystorage).
The current version of the plugin allows you to automatically encrypt MPEG DASH streams (other stream types are not currently supported).

For each stream served by the Wowza server, the plugin will assign a KID (Key Identifier) for that stream, and use a media encryption key for that stream. The media encryption key may be independtly setup on the Key Storage server, or created automatically on-the-fly the first time the stream is accessed (see below for details).
The MPEG DASH MPD generated by the Wowza server for the stream will signal the proper <ContentEncrytption> entry to indicate the DRM information needed by the client, and the server will encrypt all the media segments with the key selectd for the stream
This, then, allows you to create tokens to issue DRM licenses for that stream, using the ExpressPlay API to create either Marlin MS3 URLs or Marlin BB license acquisition tokens.

In this version of the plugin, the KID for a stream is automatically computed as follows:
KID = <first 16 bytes of SHA1(streamName)>
For example, if the stream name is sample.mp4, then the KID is 0507b9fc965d62b252252bc3818a161d

You can decide to either:
a) create key entries in advance in the ExpressPlay Key Store for each streamName/KID that you will serve using the Wowza server. In that case, the plugin will simply use the existing keys you have created
or
b) let the plugin automatically create a new key for each stream the first time it encounters it. In that case, you will see the key entries in the ExpressPlay Key Store with the 'info' field set to indicate the plugin version number, and the 'contentId' field set to the streamName.

NOTE: when using automatically-created keys, it is important to retrieve the MPEG DASH MPD at least once in order for the key to be created and stored on the server. DRM tokens can only be created after the key has been stored on the server.


Installation
------------

NOTE: Wowza Streaming Engine 4.2 or later is required.

Copy the plugin file wms-plugin-commonencryption-expressplay-0.5.jar to the server's 'lib' folder ([install-dir]/lib).
Restart the server (not just individual applications).

KNOWN ISSUE: the first time the plugin is loaded in a freshly started server instance, you may see an error message in the logs saying:
loadModFunctions[DrmVodTest/_definst_]: Module class not found or could not be loaded. Check [install-dir]/conf/DrmVodTest/Application.xml to be sure all Modules/Module/Class paths are correct: name:ExpressPlayCenc class:com.intertrust.expressplay.wms.commonencryption.ExpressPlayCencModule
If that happens, simply restart the server once, and the module will correctly load from then on.

Configuration
-------------

The plugin can be configured by editing your application's configuration either through the Web UI, or directly by editing the application's configuration XML file.
Through the settings, you will need to:
1/ Enable the plugin module
2/ Configure the plugin properties

The 3 properties that need to be set are:
  * expressPlayKeyServerUrl : use 'https://api.test.expressplay.com/keystore' when working in 'Test' mode, or 'https://api.service.expressplay.com/keystore' in 'Production' mode

  * expressPlayKeyServerKek : <32-character hex Key Encryption Key you want to use with the Key Store>

  * expressPlayKeyServerAuthenticator : <your ExpressPlay test or production Customer Authenticator, which you can obtain from the ExpressPlay dashboard>

  Using the Web UI
  ################

  NOTE: Access to the Modules tab is limited to administrators with advanced permissions.
  In the Server tab, go to Users tab, select the name of the user and click the EDIT button to allow access to advanced properties and features which is part of the Preferences section. Once this is selected, you should see the Modules and Properties tab.)
  For more information, see 'Manage credentials / Enable access to advanced administrative settings' in the Wowza documentation: http://www.wowza.com/forums/content.php?625-How-to-get-started-as-a-Wowza-Streaming-Engine-Manager-administrator#signinManager.

  On your VOD or Live application page, click the 'Modules' tab.
  On the 'Modules' tab, click the 'Edit' button.
  Click the 'Add Module' button and add an entry for the plugin module:
  Name: ExpressPlayCenc
  Description: ExpressPlay CENC Plugin
  Fully Qualified Class Name: com.intertrust.expressplay.wms.commonencryption.ExpressPlayCencModule

  Click 'Save'.

  Click the 'Properties' Tab
  Click the 'Edit' button in the 'Custom' properties section.
  Click the 'Add Custom Property' button and add the following properties 

  Path: /Root/Application
  Name: expressPlayKeyServerUrl
  Type: String
  Value: <see comment above>

  Path: /Root/Application
  Name: expressPlayKeyServerKek
  Type: String
  Value: <see comment above>

  Path: /Root/Application
  Name: expressPlayKeyServerAuthenticator
  Type: String
  Value: <see comment above>

  Click 'Save'.

  Restart the application.

  Using the Config Files directly
  ###############################
  
  Add the following <Module> entry as the last entry in the <Modules> list:
  
  <Module>
    <Name>ExpressPlayCenc</Name>
    <Description>ExpressPlay CENC Plugin</Description>
    <Class>com.intertrust.expressplay.wms.commonencryption.ExpressPlayCencModule</Class>
  </Module>

  Add the following properties to the <Properties> container at the bottom of Application.xml (be sure to get the correct <Properties> container - there are several in Application.xml):

  (replace the "***** see comment above for value ******" placeholders with the property values as described above)

  <Property>
    <Name>expressPlayKeyServerUrl</Name>
    <Value>***** see comment above for value ******</Value>
    <Type>String</Type>
  </Property>

  <Property>
    <Name>expressPlayKeyServerKek</Name>
    <Value>***** see comment above for value ******</Value>
    <Type>String</Type>
  </Property>

  <Property>
    <Name>expressPlayKeyServerAuthenticator</Name>
    <Value>***** see comment above for value ******</Value>
    <Type>String</Type>
  </Property>
  
Troubleshooting
---------------

If you encounter issues with the plugin, the first thing to do is to look at the Wowza server logs, either on stdout or in the log files generated by the server.
You should see log entries from the plugin. With the default Wowza server log4j configuration, you should see only a limited number of messages (INFO level). You can increase the logging output of the plugin by editing the conf/log4j.properties file, adding one like as follows:

log4j.logger.com.intertrust.expressplay.wms.commonencryption=DEBUG

The first log output from the plugin should be a line specifying the plugin version number. The line should look something like this:

INFO server comment - ExpressPlayCencModule: onAppStart: version=0.1.40

Example
-------

In this example, we have created a simple Wowza VOD application called DrmVodTest, with one media file called sample.mp4
We are using the 'Test' mode of the ExpressPlay service.
(Please note that the customerAuthenticator value used here is just an example, you would have to use your own customerAuthenticator. Also, DO NOT use this example Server KEK in a production service, as it would compromise the security of your keys, given that this value isn't secret)

Our application Custom Properties are:
<Properties>
    <Property>
        <Name>expressPlayKeyServerUrl</Name>
        <Value>https://api.test.expressplay.com/keystore</Value>
        <Type>String</Type>
    </Property>
    <Property>
        <Name>expressPlayKeyServerKek</Name>
        <Value>00112233445566778899aabbccddeeff</Value>
        <Type>String</Type>
    </Property>
    <Property>
        <Name>expressPlayKeyServerAuthenticator</Name>
        <Value>927,15fb72227a554245b96017b101ef86a5</Value>
        <Type>String</Type>
    </Property>
</Properties>

The MPEG DASH URL for the stream, on the example server (in this example, the server is hosted at http://my.example.server.com, on port 1935):
http://my.example.server.com:1935/DrmVodTest/mp4:sample.mp4/manifest.mpd

[in this example, we use the 'curl' command line app to retrieve HTTP URLs and print the resulting resource. Any other HTTP client could be used here]

Once we retrieve the MPEG DASH MPD from the Wowza server (to trigger the automatic creation and registration of the key for this stream), we can check that the MPD has the expected <ContentEncryption> entry, and that the key is indeed present on the server:

curl http://my.example.server.com:1935/DrmVodTest/mp4:sample.mp4/manifest.mpd
--> should show an MPD XML document that contains:
...
<ContentProtection schemeIdUri="urn:uuid:5E629AF5-38DA-4063-8977-97FFBD9902D4" value="Marlin"/>
...

to check that the key is on the server:
curl "https://api.test.expressplay.com/keystore/keys/0507b9fc965d62b252252bc3818a161d?customerAuthenticator=927,15fb72227a554245b96017b101ef86a5"
--> should show the Key entry:
{"kid":"0507B9FC965D62B252252BC3818A161D","kekId":"#1.6AB55CDD522CED2BA2652B93EF0856443551C863","ek":"7FBB5D5FE06D7646AE2B989A2E2D6C5E29A0B93530620457","info":"Wowza Plugin: streamName=sample.mp4, pluginVersion=0.1.40","contentId":"urn:marlin:kid:0507b9fc965d62b252252bc3818a161d","lastUpdate":"2014-12-15T21:24:10Z"}

We can create an MS3 URL token for this, by using the ExpressPlay REST API:
curl "https://ms3-gen.test.expressplay.com/hms/ms3/token?customerAuthenticator=927,15fb72227a554245b96017b101ef86a5&contentId=urn:marlin:kid:0507b9fc965d62b252252bc3818a161d&kid=0507b9fc965d62b252252bc3818a161d&kek=00112233445566778899aabbccddeeff&contentURL=http://my.example.server.com:1935/DrmVodTest/mp4:sample.mp4/manifest.mpd&ms3Scheme=true"

Which gives us (don't use this token, it is just an example):
ms3://ms3.test.expressplay.com:8443/hms/ms3/rights/?b=ABUAAwAAAzsAC2dpbGxlc3Rlc3QxXVwAEPdrYUyDSLDbdAEqBB1gGRIAcAp6B4VpNwXv1P9LwAhWC7T7B3B7FS_FfibWUQVgs9C9R5X0ffRqBsw2dplDXyZ56cOz4rYs9bWJhVwMfrAXsvkjqdtfggDRr0j-lCGiZGvziiZatOdZkp3TG_j8DUPEVyMOAUygjLKeMzztbkzGf7YAAAAUlVUs_06Xm9xUFrtfpDZI2pSqUUg#http%3A%2F%2Fmy.example.server.com%2FDrmVodTest%2Fmp4%3Asample.mp4%2Fmanifest.mpd

We can now try playing this ms3:// URL with the ExpressPlay sample player application

